use crate::library::track::Track;
use crate::library::Libary;
use based::auth::User;
use based::request::api::{vec_to_api, FallibleApiResponse};
use based::{check_admin, get_pg};
use rocket::{get, State};
use serde_json::json;

#[get("/library/clean")]
pub async fn clean_library(lib: &State<Libary>, u: User) -> FallibleApiResponse {
    check_admin!(u);
    lib.clean_lost_files().await;
    Ok(json!({"ok": 1}))
}

#[get("/library/singles")]
pub async fn get_singles_route(u: User) -> FallibleApiResponse {
    check_admin!(u);
    let singles: Vec<Track> =
        sqlx::query_as("SELECT * FROM track WHERE album IS NULL AND artist IS NOT NULL")
            .fetch_all(get_pg!())
            .await
            .unwrap();

    Ok(json!(vec_to_api(&singles).await))
}

#[get("/library/orphans")]
pub async fn get_orphans_route(u: User) -> FallibleApiResponse {
    check_admin!(u);
    let orphans = Track::get_orphans().await;
    Ok(json!(vec_to_api(&orphans).await))
}
